@@ -83,9 +83,9 @@ EOERRMSG;
     exit;
   }
 
-  $result = pg_exec( $dbconn, "SELECT regexp_replace( version(), E'^PostgreSQL ([0-9]+\.[0-9]+)\..*$', E'\\1')" );
+  $result = pg_exec( $dbconn, "SELECT version()" );
   $row = pg_fetch_array($result, 0);
-  $c->found_dbversion = $row[0];
+  $c->found_dbversion = preg_replace( '/^PostgreSQL (\d+\.\d+)\..*$/i', '$1', $row[0]);
 }
 
 
@@ -150,7 +150,7 @@ function qpg($str = null) {
       //PostgreSQL treats a backslash as an escape character.
       $str = str_replace('\\', '\\\\', $str);
       $rv = "'$str'";
-      if ( $c->found_dbversion > 8.0 ) $rv = 'E'.$rv;
+      if ( !isset($c->found_dbversion) || $c->found_dbversion > 8.0 ) $rv = 'E'.$rv;
   }
   return $rv;
 }
@@ -457,11 +457,8 @@ class PgQuery
       $this->_log_error( $this->location, 'DBGQ', $this->querystring, $line, $file );
     }
 
-    if ( ! (isset($c->dbg['ALL']) || isset($c->dbg['querystring']) || ini_get('display_errors') == 0 ) ) {
-      ini_set('display_errors',0);
-    }
     $t1 = microtime(); // get start time
-    $this->result = pg_exec( $this->connection, $this->querystring ); // execute the query
+    $this->result = @pg_exec( $this->connection, $this->querystring ); // execute the query
     $this->rows = ($this->result ? pg_numrows($this->result) : -1); // number of rows returned
     $t2 = microtime(); // get end time
     $i_took = duration( $t1, $t2 );   // calculate difference
@@ -616,4 +613,3 @@ class PgQuery
 
 }
 
-?>
\ No newline at end of file
