@@ -9,8 +9,8 @@
 *
 * @package   awl
 * @subpackage   Browser
-* @author    Andrew McMillan <andrew@catalyst.net.nz>
-* @copyright Catalyst IT Ltd
+* @author    Andrew McMillan <andrew@mcmillan.net.nz>
+* @copyright Catalyst IT Ltd, Morphoss Ltd <http://www.morphoss.com/>
 * @license   http://gnu.org/copyleft/gpl.html GNU GPL v2
 */
 require_once("AWLUtilities.php");
@@ -20,38 +20,6 @@ require_once("AWLUtilities.php");
 */
 $BrowserCurrentRow = (object) array();
 
-/**
-* Return values from the current row for replacing into a template.
-*
-* This is used to return values from the current row, so they can
-* be inserted into a row template.  It is used as a callback
-* function for preg_replace_callback.
-*
-* @param array of string $matches An array containing a field name as offset 1
-*/
-function BrowserColumnValueReplacement($matches)
-{
-/**
-* @global object $BrowserCurrentRow  The row most recently read from the database.
-*/
-  global $BrowserCurrentRow;
-  // as usual: $matches[0] is the complete match
-  // $matches[1] the match for the first subpattern
-  // enclosed in '##...##' and so on
-  // Use like: $s = preg_replace_callback("/##([^#]+)##/", "BrowserColumnValueReplacement", $s);
-  // $BrowserCurrentRow needs to be assigned something relevant first...
-
-  $field_name = $matches[1];
-  if ( !isset($BrowserCurrentRow->{$field_name}) && substr($field_name,0,4) == "URL:" ) {
-    $field_name = substr($field_name,4);
-    $replacement = urlencode($BrowserCurrentRow->{$field_name});
-  }
-  else {
-    $replacement = $BrowserCurrentRow->{$field_name};
-  }
-  dbg_error_log( "Browser", ":BrowserColumnValueReplacement: Replacing %s with %s", $field_name, $replacement);
-  return $replacement;
-}
 
 
 /**
@@ -130,15 +98,16 @@ class BrowserColumn
   * @param string order_direction Whether the sort is Ascending or Descending.
   * @param int browser_array_key Used this to help handle separate ordering of
   *                              multiple browsers on the same page.
+  * @param string forced_order If true, then we don't allow order to be changed.
   */
-  function RenderHeader( $order_field, $order_direction, $browser_array_key=0 ) {
+  function RenderHeader( $order_field, $order_direction, $browser_array_key=0, $forced_order=false ) {
     global $c;
     if ( $this->Align == "" ) $this->Align = "left";
     $html = '<th class="'.$this->Align.'" '. ($this->Class == "" ? "" : "class=\"$this->Class\"") . '>';
 
     $direction = 'A';
     $image = "";
-    if ( $order_field == $this->Field ) {
+    if ( !$forced_order && $order_field == $this->Field ) {
       if ( strtoupper( substr( $order_direction, 0, 1) ) == 'A' ) {
         $image = 'down';
         $direction = 'D';
@@ -149,9 +118,10 @@ class BrowserColumn
       $image = "<img class=\"order\" src=\"$c->images/$image.gif\" alt=\"$image\" />";
     }
     if ( !isset($browser_array_key) || $browser_array_key == '' ) $browser_array_key = 0;
-    $html .= '<a href="'.replace_uri_params( $_SERVER['REQUEST_URI'], array( "o[$browser_array_key]" => $this->Field, "d[$browser_array_key]" => $direction ) ).'" class="order">';
+    if ( !$forced_order ) $html .= '<a href="'.replace_uri_params( $_SERVER['REQUEST_URI'], array( "o[$browser_array_key]" => $this->Field, "d[$browser_array_key]" => $direction ) ).'" class="order">';
     $html .= ($this->Header == "" ? $this->Field : $this->Header);
-    $html .= "$image</a></th>\n";
+    if ( !$forced_order ) $html .= "$image</a>";
+    $html .= "</th>\n";
     return $html;
   }
 
@@ -161,7 +131,6 @@ class BrowserColumn
 
   function RenderValue( $value, $extraclass = "" ) {
     global $session;
-    global $BrowserCurrentRow;
 
     if ( $this->Type == 'date' || $this->Type == 'timestamp') {
       $value = $session->FormattedDate( $value, $this->Type );
@@ -169,7 +138,7 @@ class BrowserColumn
 
     if ( $this->Hook && function_exists($this->Hook) ) {
       dbg_error_log( "Browser", ":Browser: Hook for $this->Hook on column $this->Field");
-      $value = call_user_func( $this->Hook, $value, $this->Field, $BrowserCurrentRow );
+      $value = call_user_func( $this->Hook, $value, $this->Field, $this->current_row );
     }
 
     if ( $this->Translatable ) {
@@ -189,7 +158,6 @@ class BrowserColumn
       $html .= ($this->Format == "" ? $value : sprintf($this->Format,$value,$value));
       $html .= "</td>\n";
     }
-    $html = preg_replace_callback("/##([^#]+)##/", "BrowserColumnValueReplacement", $html );
     return $html;
   }
 }
@@ -211,13 +179,16 @@ class Browser
   var $HiddenColumns;
   var $Joins;
   var $Where;
+  var $Distinct;
   var $Union;
   var $Order;
   var $OrderField;
   var $OrderDirection;
   var $OrderBrowserKey;
+  var $ForcedOrder;
   var $Grouping;
   var $Limit;
+  var $Offset;
   var $Query;
   var $BeginRow;
   var $CloseRow;
@@ -228,6 +199,8 @@ class Browser
   var $match_column;
   var $match_value;
   var $match_function;
+  var $DivOpen;
+  var $DivClose;
 
   /**
   * The Browser class constructor
@@ -238,8 +211,10 @@ class Browser
     global $c;
     $this->Title = $title;
     $this->SubTitle = "";
+    $this->Distinct = "";
     $this->Order = "";
     $this->Limit = "";
+    $this->Offset = "";
     $this->BeginRow = "<tr class=\"row%d\">\n";
     $this->CloseRow = "</tr>\n";
     $this->BeginRowArgs = array('#even');
@@ -247,6 +222,9 @@ class Browser
     $this->Columns = array();
     $this->HiddenColumns = array();
     $this->FieldNames = array();
+    $this->DivOpen = '<div id="browser">';
+    $this->DivClose = '</div>';
+    $this->ForcedOrder = false;
     dbg_error_log( "Browser", ":Browser: New browser called $title");
   }
 
@@ -335,6 +313,17 @@ class Browser
     $this->SubTitle = $sub_title;
   }
 
+  /**
+  * Set a div for wrapping the browse.
+  *
+  * @param string $open_div The HTML to open the div
+  * @param string $close_div The HTML to open the div
+  */
+  function SetDiv( $open_div, $close_div ) {
+    $this->DivOpen = $open_div;
+    $this->DivClose = $close_div;
+  }
+
   /**
   * Set the tables and joins for the SQL.
   *
@@ -372,6 +361,39 @@ class Browser
     $this->Where = $where_clause;
   }
 
+  /**
+  * Set the SQL DISTINCT clause to a specific value.
+  *
+  * The whole clause (except the keyword) needs to be supplied
+  *
+  * @param string $distinct The whole clause, after 'DISTINCT'
+  */
+  function SetDistinct( $distinct ) {
+    $this->Distinct = "DISTINCT ".$distinct;
+  }
+
+  /**
+  * Set the SQL LIMIT clause to a specific value.
+  *
+  * Only the limit number should be supplied.
+  *
+  * @param int $limit_n A number of rows to limit the SQL selection to
+  */
+  function SetLimit( $limit_n ) {
+    $this->Limit = "LIMIT ".intval($limit_n);
+  }
+
+  /**
+  * Set the SQL OFFSET clause to a specific value.
+  *
+  * Only the offset number
+  *
+  * @param int $offset_n A number of rows to offset the SQL selection to, based from the start of the results.
+  */
+  function SetOffset( $offset_n ) {
+    $this->Offset = "OFFSET ".intval($offset_n);
+  }
+
   /**
   * Add an [operator] ... to the SQL Where clause
   *
@@ -433,11 +455,6 @@ class Browser
   *               which shouldn't interfere with the default primary order.
   */
   function AddOrder( $field, $direction, $browser_array_key=0, $secondary=0 ) {
-    if ( $secondary == 0 && ( is_array($field) || is_array($direction) ) ) {
-      if ( $browser_array_key >= count($field) || $browser_array_key >= count($direction) ) return;
-      $field = $field[$browser_array_key];
-      $direction = $direction[$browser_array_key];
-    }
     $field = clean_string($field);
     if ( ! isset($this->FieldNames[$field]) ) return;
 
@@ -465,6 +482,50 @@ class Browser
   }
 
 
+  /**
+  * Force a particular ordering onto the browser widget.
+  *
+  * @param string $field The name of the field to be ordered by.
+  * @param string $direction A for Ascending, otherwise it will be descending order.
+  */
+  function ForceOrder( $field, $direction ) {
+    $field = clean_string($field);
+    if ( ! isset($this->FieldNames[$field]) ) return;
+
+    if ( $this->Order == "" )
+      $this->Order = "ORDER BY ";
+    else
+      $this->Order .= ", ";
+
+    $this->Order .= $field;
+
+    if ( preg_match( '/^A/i', $direction) ) {
+      $this->Order .= " ASC";
+    }
+    else {
+      $this->Order .= " DESC";
+    }
+
+    $this->ForcedOrder = true;
+  }
+
+
+  /**
+  * Set up the ordering for the browser.  Generally you should call this with
+  * the first parameter set as a field to order by default.  Call with the second
+  * parameter set to 'D' or 'DESCEND' if you want to reverse the default order.
+  */
+  function SetOrdering( $default_fld=null, $default_dir='A' , $browser_array_key=0 ) {
+    if ( isset( $_GET['o'][$browser_array_key] ) && isset($_GET['d'][$browser_array_key] ) ) {
+      $this->AddOrder( $_GET['o'][$browser_array_key], $_GET['d'][$browser_array_key], $browser_array_key );
+    }
+    else {
+      if ( ! isset($default_fld) ) $default_fld = $this->Columns[0];
+      $this->AddOrder( $default_fld, $default_dir, $browser_array_key );
+    }
+  }
+
+
   /**
   * Mark a column as something to be totalled.  You can also specify the name of
   * a function which may modify the value before the actual totalling.
@@ -484,6 +545,16 @@ class Browser
   }
 
 
+  /**
+  * Retrieve the total from a totalled column
+  *
+  * @param string $column_name The name of the column to be totalled.
+  */
+  function GetTotal( $column_name ) {
+    return $this->Totals[$column_name];
+  }
+
+
   /**
   * Set the format for an output row.
   *
@@ -540,12 +611,12 @@ class Browser
       }
     }
     $where_clause = ((isset($this->Where) && $this->Where != "") ? "WHERE $this->Where" : "" );
-    $sql = sprintf( "SELECT %s FROM %s %s %s ", $target_fields,
+    $sql = sprintf( "SELECT %s %s FROM %s %s %s ", $this->Distinct, $target_fields,
                  $this->Joins, $where_clause, $this->Grouping );
     if ( "$this->Union" != "" ) {
       $sql .= "UNION $this->Union ";
     }
-    $sql .= $this->Order . ' ' . $this->Limit;
+    $sql .= $this->Order . ' ' . $this->Limit . ' ' . $this->Offset;
     $this->Query = new PgQuery( $sql );
     return $this->Query->Exec("Browse:$this->Title:DoQuery");
   }
@@ -576,6 +647,34 @@ class Browser
   }
 
 
+  /**
+  * Return values from the current row for replacing into a template.
+  *
+  * This is used to return values from the current row, so they can
+  * be inserted into a row template.  It is used as a callback
+  * function for preg_replace_callback.
+  *
+  * @param array of string $matches An array containing a field name as offset 1
+  */
+  function ValueReplacement($matches)
+  {
+    // as usual: $matches[0] is the complete match
+    // $matches[1] the match for the first subpattern
+    // enclosed in '##...##' and so on
+
+    $field_name = $matches[1];
+    if ( !isset($this->current_row->{$field_name}) && substr($field_name,0,4) == "URL:" ) {
+      $field_name = substr($field_name,4);
+      $replacement = urlencode($this->current_row->{$field_name});
+    }
+    else {
+      $replacement = (isset($this->current_row->{$field_name}) ? $this->current_row->{$field_name} : '');
+    }
+    dbg_error_log( "Browser", ":ValueReplacement: Replacing %s with %s", $field_name, $replacement);
+    return $replacement;
+  }
+
+
   /**
   * This method is used to render the browser as HTML.  If the query has
   * not yet been executed then this will call DoQuery to do so.
@@ -586,24 +685,26 @@ class Browser
   * @param string $title_tag The tag to use around the browser title (default 'h1')
   * @return string The rendered HTML fragment to display to the user.
   */
-  function Render( $title_tag = 'h1', $subtitle_tag = 'h2' ) {
-    global $c, $BrowserCurrentRow;
+  function Render( $title_tag = null, $subtitle_tag = null ) {
+    global $c;
 
     if ( !isset($this->Query) ) $this->DoQuery();  // Ensure the query gets run before we render!
 
     dbg_error_log( "Browser", ":Render: browser $this->Title");
-    $html = '<div id="browser">';
+    $html = $this->DivOpen;
     if ( $this->Title != "" ) {
+      if ( !isset($title_tag) ) $title_tag = 'h1';
       $html .= "<$title_tag>$this->Title</$title_tag>\n";
     }
     if ( $this->SubTitle != "" ) {
+      if ( !isset($subtitle_tag) ) $subtitle_tag = 'h2';
       $html .= "<$subtitle_tag>$this->SubTitle</$subtitle_tag>\n";
     }
 
     $html .= "<table id=\"browse_table\">\n";
     $html .= "<thead><tr class=\"header\">\n";
     foreach( $this->Columns AS $k => $column ) {
-      $html .= $column->RenderHeader( $this->OrderField, $this->OrderDirection, $this->OrderBrowserKey );
+      $html .= $column->RenderHeader( $this->OrderField, $this->OrderDirection, $this->OrderBrowserKey, $this->ForcedOrder );
     }
     $html .= "</tr></thead>\n<tbody>";
 
@@ -625,15 +726,15 @@ class Browser
         }
       }
       // Start the row
-      $html .= vsprintf( $this->BeginRow, $rowanswers);
+      $row_html = vsprintf( $this->BeginRow, $rowanswers);
 
       if ( isset($this->match_column) && isset($this->match_value) && $BrowserCurrentRow->{$this->match_column} == $this->match_value ) {
-        $html .= call_user_func( $this->match_function, $BrowserCurrentRow );
+        $row_html .= call_user_func( $this->match_function, $BrowserCurrentRow );
       }
       else {
         // Each column
         foreach( $this->Columns AS $k => $column ) {
-          $html .= $column->RenderValue($BrowserCurrentRow->{$column->Field});
+          $row_html .= $column->RenderValue( (isset($BrowserCurrentRow->{$column->Field})?$BrowserCurrentRow->{$column->Field}:'') );
           if ( isset($this->Totals[$column->Field]) ) {
             if ( isset($this->TotalFuncs[$column->Field]) && function_exists($this->TotalFuncs[$column->Field]) ) {
               // Run the amount through the callback function  $floatval = my_function( $row, $fieldval );
@@ -641,28 +742,32 @@ class Browser
             }
             else {
               // Just add the amount
-              $this->Totals[$column->Field] += $BrowserCurrentRow->{$column->Field};
+              $this->Totals[$column->Field] += doubleval( preg_replace( '/[^0-9.-]/', '', $BrowserCurrentRow->{$column->Field} ));
             }
           }
         }
       }
 
       // Finish the row
-      $html .= $this->CloseRow;
+      $row_html .= $this->CloseRow;
+      $this->current_row = $BrowserCurrentRow;
+      $html .= preg_replace_callback("/##([^#]+)##/", array( &$this, "ValueReplacement"), $row_html );
     }
 
     if ( count($this->Totals) > 0 ) {
       $BrowserCurrentRow = (object) "";
-      $html .= "<tr class=\"totals\">\n";
+      $row_html = "<tr class=\"totals\">\n";
       foreach( $this->Columns AS $k => $column ) {
         if ( isset($this->Totals[$column->Field]) ) {
-          $html .= $column->RenderValue( $this->Totals[$column->Field], "totals" );
+          $row_html .= $column->RenderValue( $this->Totals[$column->Field], "totals" );
         }
         else {
-          $html .= $column->RenderValue( "" );
+          $row_html .= $column->RenderValue( "" );
         }
       }
-      $html .= "</tr>\n";
+      $row_html .= "</tr>\n";
+      $this->current_row = $BrowserCurrentRow;
+      $html .= preg_replace_callback("/##([^#]+)##/", array( &$this, "ValueReplacement"), $row_html );
     }
 
 
@@ -686,25 +791,27 @@ class Browser
         }
 
         // Start the row
-        $html .= vsprintf( $this->BeginRow, $rowanswers);
+        $row_html = vsprintf( $this->BeginRow, $rowanswers);
 
         if ( isset($this->match_column) && isset($this->match_value) && $BrowserCurrentRow->{$this->match_column} == $this->match_value ) {
-          $html .= call_user_func( $this->match_function, $BrowserCurrentRow );
+          $row_html .= call_user_func( $this->match_function, $BrowserCurrentRow );
         }
         else {
           // Each column
           foreach( $this->Columns AS $k => $column ) {
-            $html .= $column->RenderValue( (isset($BrowserCurrentRow->{$column->Field}) ? $BrowserCurrentRow->{$column->Field} : '') );
+            $row_html .= $column->RenderValue( (isset($BrowserCurrentRow->{$column->Field}) ? $BrowserCurrentRow->{$column->Field} : '') );
           }
         }
 
         // Finish the row
-        $html .= $this->CloseRow;
+        $row_html .= $this->CloseRow;
+        $this->current_row = $BrowserCurrentRow;
+        $html .= preg_replace_callback("/##([^#]+)##/", array( &$this, "ValueReplacement"), $row_html );
       }
     }
 
     $html .= "</tbody>\n</table>\n";
-    $html .= '</div>';
+    $html .= $this->DivClose;
 
     return $html;
   }
