@@ -133,7 +133,7 @@ class iCalProp {
       $value = substr( $v, $pos + 1);
       $this->parameters[$name] = $value;
     }
-    dbg_error_log("iCalendar", " iCalProp::ParseFrom found '%s' = '%s' with %d parameters", $this->name, $this->content, count($this->parameters) );
+    dbg_error_log("iCalendar", " iCalProp::ParseFrom found '%s' = '%s' with %d parameters", $this->name, substr($this->content,0,200), count($this->parameters) );
   }
 
 
@@ -335,7 +335,7 @@ class iCalComponent {
     $this->properties = array();
     $this->components = array();
     $this->rendered = "";
-    if ( $content != null && gettype($content) == 'string' ) {
+    if ( $content != null && (gettype($content) == 'string' || gettype($content) == 'array') ) {
       $this->ParseFrom($content);
     }
   }
@@ -386,17 +386,32 @@ class iCalComponent {
     $this->rendered = $content;
     $content = $this->UnwrapComponent($content);
 
-    $lines = preg_split('/\r?\n/', $content );
-
     $type = false;
     $subtype = false;
     $finish = null;
     $subfinish = null;
-    foreach( $lines AS $k => $v ) {
-      if ( preg_match('/^\s*$/', $v ) ) continue;
-      dbg_error_log( "iCalendar",  "::ParseFrom: Parsing line: $v");
+
+    $length = strlen($content);
+    $linefrom = 0;
+    while( $linefrom < $length ) {
+      $lineto = strpos( $content, "\n", $linefrom );
+      if ( $lineto === false ) {
+        $lineto = strpos( $content, "\r", $linefrom );
+      }
+      if ( $lineto > 0 ) {
+        $line = substr( $content, $linefrom, $lineto - $linefrom);
+        $linefrom = $lineto + 1;
+      }
+      else {
+        $line = substr( $content, $linefrom );
+        $linefrom = $length;
+      }
+      if ( preg_match('/^\s*$/', $line ) ) continue;
+      $line = rtrim( $line, "\r\n" );
+      dbg_error_log( "iCalendar",  "::ParseFrom: Parsing line: $line");
+
       if ( $type === false ) {
-        if ( preg_match( '/^BEGIN:(.+)$/', $v, $matches ) ) {
+        if ( preg_match( '/^BEGIN:(.+)$/', $line, $matches ) ) {
           // We have found the start of the main component
           $type = $matches[1];
           $finish = "END:$type";
@@ -404,32 +419,32 @@ class iCalComponent {
           dbg_error_log( "iCalendar", "::ParseFrom: Start component of type '%s'", $type);
         }
         else {
-          dbg_error_log( "iCalendar", "::ParseFrom: Ignoring crap before start of component");
-          unset($lines[$k]);  // The content has crap before the start
-          if ( $v != "" ) $this->rendered = null;
+          dbg_error_log( "iCalendar", "::ParseFrom: Ignoring crap before start of component: $line");
+          // unset($lines[$k]);  // The content has crap before the start
+          if ( $line != "" ) $this->rendered = null;
         }
       }
       else if ( $type == null ) {
         dbg_error_log( "iCalendar", "::ParseFrom: Ignoring crap after end of component");
         unset($lines[$k]);  // The content has crap after the end
-        if ( $v != "" ) $this->rendered = null;
+        if ( $line != "" ) $this->rendered = null;
       }
-      else if ( $v == $finish ) {
+      else if ( $line == $finish ) {
         dbg_error_log( "iCalendar", "::ParseFrom: End of component");
         $type = null;  // We have reached the end of our component
       }
       else {
-        if ( $subtype === false && preg_match( '/^BEGIN:(.+)$/', $v, $matches ) ) {
+        if ( $subtype === false && preg_match( '/^BEGIN:(.+)$/', $line, $matches ) ) {
           // We have found the start of a sub-component
           $subtype = $matches[1];
           $subfinish = "END:$subtype";
-          $subcomponent = "$v\r\n";
+          $subcomponent = $line . "\r\n";
           dbg_error_log( "iCalendar", "::ParseFrom: Found a subcomponent '%s'", $subtype);
         }
         else if ( $subtype ) {
           // We are inside a sub-component
-          $subcomponent .= $this->WrapComponent($v);
-          if ( $v == $subfinish ) {
+          $subcomponent .= $this->WrapComponent($line);
+          if ( $line == $subfinish ) {
             dbg_error_log( "iCalendar", "::ParseFrom: End of subcomponent '%s'", $subtype);
             // We have found the end of a sub-component
             $this->components[] = new iCalComponent($subcomponent);
@@ -441,7 +456,7 @@ class iCalComponent {
         else {
           dbg_error_log( "iCalendar", "::ParseFrom: Parse property of component");
           // It must be a normal property line within a component.
-          $this->properties[] = new iCalProp($v);
+          $this->properties[] = new iCalProp($line);
         }
       }
     }
@@ -695,10 +710,12 @@ class iCalComponent {
   * @param array of iCalComponent $new_components The new components to replace the existing ones
   * @param string $type The type of components to be replaced.  Defaults to null, which means all components will be replaced.
   */
-  function SetComponents( $new_components, $type = null ) {
+  function SetComponents( $new_component, $type = null ) {
     if ( isset($this->rendered) ) unset($this->rendered);
-    if ( count($new_components) > 0 ) $this->ClearComponents($type);
-    $this->components = $this->components + $new_components;
+    if ( count($new_component) > 0 ) $this->ClearComponents($type);
+    foreach( $new_component AS $k => $v ) {
+      $this->components[] = $v;
+    }
   }
 
 
@@ -721,6 +738,68 @@ class iCalComponent {
   }
 
 
+  /**
+  * Mask components, removing any that are not of the types in the list
+  * @param array $keep An array of component types to be kept
+  */
+  function MaskComponents( $keep ) {
+    foreach( $this->components AS $k => $v ) {
+      if ( ! in_array( $v->GetType(), $keep ) ) {
+        unset($this->components[$k]);
+        if ( isset($this->rendered) ) unset($this->rendered);
+      }
+      else {
+        $v->MaskComponents($keep);
+      }
+    }
+  }
+
+
+  /**
+  * Mask properties, removing any that are not in the list
+  * @param array $keep An array of property names to be kept
+  * @param array $component_list An array of component types to check within
+  */
+  function MaskProperties( $keep, $component_list=null ) {
+    foreach( $this->components AS $k => $v ) {
+      $v->MaskProperties($keep, $component_list);
+    }
+
+    if ( !isset($component_list) || in_array($this->GetType(),$component_list) ) {
+      foreach( $this->components AS $k => $v ) {
+        if ( ! in_array( $v->GetType(), $keep ) ) {
+          unset($this->components[$k]);
+          if ( isset($this->rendered) ) unset($this->rendered);
+        }
+      }
+    }
+  }
+
+
+  /**
+  * Clone this component (and subcomponents) into a confidential version of it.  A confidential
+  * event will be scrubbed of any identifying characteristics other than time/date, repeat, uid
+  * and a summary which is just a translated 'Busy'.
+  */
+  function CloneConfidential() {
+    $confidential = clone($this);
+    $keep_properties = array( 'DTSTAMP', 'DTSTART', 'RRULE', 'DURATION', 'DTEND', 'UID', 'CLASS', 'TRANSP' );
+    $resource_components = array( 'VEVENT', 'VTODO', 'VJOURNAL' );
+    $confidential->MaskComponents(array( 'VTIMEZONE', 'VEVENT', 'VTODO', 'VJOURNAL' ));
+    $confidential->MaskProperties($keep_properties, $resource_components );
+    if ( in_array( $confidential->GetType(), $resource_components ) ) {
+      $confidential->AddProperty( 'SUMMARY', translate('Busy') );
+    }
+    foreach( $confidential->components AS $k => $v ) {
+      if ( in_array( $v->GetType(), $resource_components ) ) {
+        $v->AddProperty( 'SUMMARY', translate('Busy') );
+      }
+    }
+
+    return $confidential;
+  }
+
+
   /**
   *  Renders the component, possibly restricted to only the listed properties
   */
@@ -733,11 +812,12 @@ class iCalComponent {
 
     $rendered = "BEGIN:$this->type\r\n";
     foreach( $this->properties AS $k => $v ) {
-      if ( $unrestricted || isset($restricted_properties[$v]) ) $rendered .= $v->Render() . "\r\n";
+      if ( method_exists($v, 'Render') ) {
+        if ( $unrestricted || isset($restricted_properties[$v]) ) $rendered .= $v->Render() . "\r\n";
+      }
     }
     foreach( $this->components AS $v ) {   $rendered .= $v->Render();  }
     $rendered .= "END:$this->type\r\n";
-//    $rendered = $this->WrapComponent($rendered);
 
     if ( $unrestricted ) $this->rendered = $rendered;
 
